package slgui.gfx;

import slgui.base.MediaPainter;

/** A Painter object holds a graphics state which can be used for immediate rendering operations.
 * Painter objects extend the minimalist MediaPainter class with an internal context stack and
 * additional, more-convenient rendering functions.
 * 
 * @author Zak Fenton
 */
public class Painter extends MediaPainter {
	private final MediaPainter target;
	private final PainterContext[] contexts;
	private int stackIndex = -1;
	
	/* The following should have equivalents in the PainterContext class: */
	Transformer transformer = Transformer.DIRECT;

	public Painter(MediaPainter target, int stackmax) {
		this.target = target;
		contexts = new PainterContext[stackmax];
	}
	
	public PainterContext contextAt(int index) {
		if (contexts[index] == null) {
			contexts[index] = new PainterContext();
		}
		return contexts[index];
	}
	
	public void save() {
		int nsi = stackIndex + 1;
		
		PainterContext ctx = contextAt(nsi);
		ctx.save(this, nsi);
		
		stackIndex = nsi;
	}
	
	public void restore() {
		PainterContext ctx = contextAt(stackIndex);
		ctx.restore(this);
		ctx.reset();
		stackIndex = stackIndex - 1;
	}
	
	/** Applies the given transformer (over the top of any already-applied transformers). */
	public void transform(Transformer t) {
		transformer = t.over(transformer);
	}
	
	/** Applies a TranslateTransformer with the given translation offset coordinates (over the top of any already-applied transformers). */
	public void translate(double tx, double ty) {
		transform(new TranslateTransformer(tx, ty));
	}
	
	@Override
	public double snapX(double x) {
		return transformer.unX(target.snapX(transformer.x(x)));
	}
	
	@Override
	public double snapY(double y) {
		return transformer.unY(target.snapY(transformer.y(y)));
	}

	@Override
	public void line(double x1, double y1, double x2, double y2, int pixel, double radius) {
		x1 = transformer.x(x1);
		y1 = transformer.y(y1);
		x2 = transformer.x(x2);
		y2 = transformer.y(y2);
		radius = transformer.radius(radius);
		target.line(x1, y1, x2, y2, pixel, radius);
	}
	
	@Override
	public void box(double x, double y, double width, double height, int pixel) {
		x = transformer.x(x);
		y = transformer.x(x);
		width = transformer.width(width);
		height = transformer.height(height);
		target.box(x, y, width, height, pixel);
	}

}
